// use std::io::BufReader;
// use std::path::PathBuf;
// use std::thread;
// use std::time::Duration;
extern crate rodio;
mod base;
use base::*;
use imgui::*;
use walkdir::WalkDir;
mod support_ogl;
// mod support_glium;
use env_logger;
use log::*;

fn collect_sounds (root: &str) -> Vec<Sound> {
    WalkDir::new(root)
    .into_iter()
    .filter_map(|e| e.ok())
    .filter(|e| e.path().is_file())
    .map(|f| Sound::new(&f.path().to_string_lossy().to_string()))
    .filter_map(|e| e.ok())
    .collect()
}


fn main() {
    env_logger::init();
    let dev = rodio::default_output_device().unwrap();
    
    let mut sounds = collect_sounds("media");
    
    let mut timeline = Timeline::default();
    // let mut track = Track::default();
    // track.test();
    // track.bar_mut(0).bpm = 1;
    // track.bars[0].bpm = 1;
    // let mut x = &mut track.bars[0];
    // x.bpm = 1;

    // dbg!(&track);

    // return;
    let mut pattern_col: usize = 0;
    let mut pattern_row: usize = 0;
    // let mut active_track: usize = 0;
    // let mut active_bar: usize = 0;


    // let mut active_bar = Bar::default();
    // let mut active_bar = &mut timeline.patterns[0].bars[0];
    let mut active_pattern = Pattern::default();
    let mut active_sound: &Option<Sound> = &None;
    // let mut active_pattern = &mut timeline.patterns[0];


    let mut system = support_ogl::init(file!());
    let s = system.imgui.style_mut();
    s.window_rounding = 1.5;
    // let system = support_glium::init(file!());

    system.main_loop(move |_, ui| {
        Window::new(im_str!("sources"))
            // .resizable(false)
            .position([400.0, 140.0], Condition::Appearing)
            .collapsible(false)
            // .always_auto_resize(true)
            .size([300.0, 300.0], Condition::FirstUseEver)
            .build(ui, || {

                // ui.show_default_style_editor();

                ui.tree_node(im_str!("snds"))
                .opened(true, Condition::Appearing)
                .build(|| {
                    for s in &mut sounds {
                        ui.tree_node(&im_str!("{}", s.name)).build(|| {
                            // ui.same_line(0.0);
                            if ui.small_button(im_str!("add to pattern")) {
                                active_pattern.sounds[pattern_row][pattern_col] = Some(s.clone());
                            }
                            ui.same_line(0.0);
                            if ui.small_button(im_str!("play")) {
                                s.play(&dev);
                            }
                        });
                    }
                });
            });

        Window::new(im_str!("pattern"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {

                // for i in 0..active_pattern.length() {
                //     ui.same_line(0.0);
                    
                //     let token = match pattern_col == i {
                //         true => ui.push_style_color(StyleColor::Button, [1.0, 0.0, 0.0, 1.0]),
                //         false => ui.push_style_color(StyleColor::CheckMark, [0.0, 0.0, 0.0, 1.0])
                //     };
                //     if ui.small_button(&im_str!("{}", i)) {
                //         pattern_col = i as usize;
                //     }
                //     token.pop(ui);

                // }

                for (row_idx, row) in (&active_pattern).sounds.iter().enumerate() {
                    ui.text(im_str!("row {}", row_idx));
                    for (snd_idx, sound) in row.iter().enumerate() {
                        ui.same_line(0.0);
                        // ui.text(im_str!("snd {}", snd_idx));

                        let label = match sound {
                            Some(_s) => "X",
                            None => " "
                        };

                        if ui.small_button(&im_str!("{} {}x{}", label, row_idx, snd_idx)) {
                            pattern_col = snd_idx;
                            pattern_row = row_idx;
                            // pattern_col = i as usize;
                        }
                    }

                }

                ui.text(im_str!("pat {} {}", pattern_col, pattern_row));


                if ui.small_button(&im_str!("column++")) {
                    active_pattern.extend_length(1);
                }
                
                if ui.small_button(&im_str!("row++")) {
                    active_pattern.extend_row(1);
                }

                
                if ui.small_button(&im_str!("ply")) {
                    active_pattern.play(&dev);
                }
            });
    
            Window::new(im_str!("sound"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {
                // let snd = &active_bar.sounds[pattern_col];
                if let Some(s) = &active_sound {
                    ui.text(im_str!("name   {}", s.name));
                    ui.text(im_str!("roll   {}", s.roll));
                    ui.text(im_str!("volume {}", s.volume));
                    ui.text(im_str!("rate   {}", s.rate));
                }
            });
    
            Window::new(im_str!("tracks"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {
               
                if ui.small_button(&im_str!("add")) {
                    active_pattern.play(&dev);
                }
            });
    
    
        });
}
